/****************************************************************************
** Meta object code from reading C++ file 'keysmith.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../keysmith-26.07.80/src/app/keysmith.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'keysmith.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3app10NavigationE_t {};
} // unnamed namespace

template <> constexpr inline auto app::Navigation::qt_create_metaobjectdata<qt_meta_tag_ZN3app10NavigationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "app::Navigation",
        "routed",
        "",
        "app::Navigation::Page",
        "route",
        "transferred",
        "pushed",
        "popped",
        "push",
        "page",
        "modelToTransfer",
        "navigate",
        "pop",
        "name",
        "Page",
        "Error",
        "AddAccount",
        "ImportAccount",
        "RenameAccount",
        "AccountsOverview",
        "SetupPassword",
        "UnlockAccounts",
        "ScanQR"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'routed'
        QtMocHelpers::SignalData<void(app::Navigation::Page, QObject *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QObjectStar, 5 },
        }}),
        // Signal 'pushed'
        QtMocHelpers::SignalData<void(app::Navigation::Page, QObject *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QObjectStar, 5 },
        }}),
        // Signal 'popped'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'push'
        QtMocHelpers::SlotData<void(app::Navigation::Page, QObject *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { QMetaType::QObjectStar, 10 },
        }}),
        // Slot 'navigate'
        QtMocHelpers::SlotData<void(app::Navigation::Page, QObject *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { QMetaType::QObjectStar, 10 },
        }}),
        // Slot 'pop'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'name'
        QtMocHelpers::MethodData<QString(app::Navigation::Page) const>(13, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 3, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Page'
        QtMocHelpers::EnumData<enum Page>(14, 14, QMC::EnumFlags{}).add({
            {   15, Page::Error },
            {   16, Page::AddAccount },
            {   17, Page::ImportAccount },
            {   18, Page::RenameAccount },
            {   19, Page::AccountsOverview },
            {   20, Page::SetupPassword },
            {   21, Page::UnlockAccounts },
            {   22, Page::ScanQR },
        }),
    };
    return QtMocHelpers::metaObjectData<Navigation, qt_meta_tag_ZN3app10NavigationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject app::Navigation::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3app10NavigationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3app10NavigationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3app10NavigationE_t>.metaTypes,
    nullptr
} };

void app::Navigation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Navigation *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->routed((*reinterpret_cast<std::add_pointer_t<app::Navigation::Page>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 1: _t->pushed((*reinterpret_cast<std::add_pointer_t<app::Navigation::Page>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 2: _t->popped(); break;
        case 3: _t->push((*reinterpret_cast<std::add_pointer_t<app::Navigation::Page>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 4: _t->navigate((*reinterpret_cast<std::add_pointer_t<app::Navigation::Page>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 5: _t->pop(); break;
        case 6: { QString _r = _t->name((*reinterpret_cast<std::add_pointer_t<app::Navigation::Page>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Navigation::*)(app::Navigation::Page , QObject * )>(_a, &Navigation::routed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Navigation::*)(app::Navigation::Page , QObject * )>(_a, &Navigation::pushed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Navigation::*)()>(_a, &Navigation::popped, 2))
            return;
    }
}

const QMetaObject *app::Navigation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *app::Navigation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3app10NavigationE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int app::Navigation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void app::Navigation::routed(app::Navigation::Page _t1, QObject * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void app::Navigation::pushed(app::Navigation::Page _t1, QObject * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void app::Navigation::popped()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN3app8KeysmithE_t {};
} // unnamed namespace

template <> constexpr inline auto app::Keysmith::qt_create_metaobjectdata<qt_meta_tag_ZN3app8KeysmithE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "app::Keysmith",
        "copyToClipboard",
        "",
        "text",
        "accountListModel",
        "model::SimpleAccountListModel*",
        "passwordRequest",
        "model::PasswordRequest*",
        "navigation",
        "app::Navigation*"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'copyToClipboard'
        QtMocHelpers::MethodData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Method 'accountListModel'
        QtMocHelpers::MethodData<model::SimpleAccountListModel *()>(4, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Method 'passwordRequest'
        QtMocHelpers::MethodData<model::PasswordRequest *()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'navigation'
        QtMocHelpers::PropertyData<app::Navigation*>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Keysmith, qt_meta_tag_ZN3app8KeysmithE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject app::Keysmith::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3app8KeysmithE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3app8KeysmithE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3app8KeysmithE_t>.metaTypes,
    nullptr
} };

void app::Keysmith::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Keysmith *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->copyToClipboard((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: { model::SimpleAccountListModel* _r = _t->accountListModel();
            if (_a[0]) *reinterpret_cast<model::SimpleAccountListModel**>(_a[0]) = std::move(_r); }  break;
        case 2: { model::PasswordRequest* _r = _t->passwordRequest();
            if (_a[0]) *reinterpret_cast<model::PasswordRequest**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< app::Navigation* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<app::Navigation**>(_v) = _t->navigation(); break;
        default: break;
        }
    }
}

const QMetaObject *app::Keysmith::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *app::Keysmith::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3app8KeysmithE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int app::Keysmith::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
