/****************************************************************************
** Meta object code from reading C++ file 'input.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../keysmith-26.07.80/src/model/input.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'input.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5model12AccountInputE_t {};
} // unnamed namespace

template <> constexpr inline auto model::AccountInput::qt_create_metaobjectdata<qt_meta_tag_ZN5model12AccountInputE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "model::AccountInput",
        "typeChanged",
        "",
        "nameChanged",
        "issuerChanged",
        "secretChanged",
        "tokenLengthChanged",
        "timeStepChanged",
        "algorithmChanged",
        "epochChanged",
        "checksumChanged",
        "counterChanged",
        "truncationChanged",
        "reset",
        "setCounter",
        "counter",
        "validators::UnsignedLongValidator*",
        "validator",
        "setTruncationOffset",
        "truncationOffset",
        "setDynamicTruncation",
        "type",
        "model::AccountInput::TokenType",
        "name",
        "issuer",
        "secret",
        "tokenLength",
        "timeStep",
        "algorithm",
        "model::AccountInput::TOTPAlgorithm",
        "epoch",
        "checksum",
        "fixedTruncation",
        "TOTPAlgorithm",
        "Sha1",
        "Sha256",
        "Sha512",
        "TokenType",
        "Hotp",
        "Totp"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'typeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'issuerChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'secretChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tokenLengthChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'timeStepChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'algorithmChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'epochChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'checksumChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'counterChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'truncationChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'reset'
        QtMocHelpers::MethodData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'setCounter'
        QtMocHelpers::MethodData<void(const QString &, validators::UnsignedLongValidator *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Method 'setTruncationOffset'
        QtMocHelpers::MethodData<void(uint)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 19 },
        }}),
        // Method 'setDynamicTruncation'
        QtMocHelpers::MethodData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<model::AccountInput::TokenType>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'issuer'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'secret'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'tokenLength'
        QtMocHelpers::PropertyData<uint>(26, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'timeStep'
        QtMocHelpers::PropertyData<uint>(27, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'algorithm'
        QtMocHelpers::PropertyData<model::AccountInput::TOTPAlgorithm>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'epoch'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'checksum'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'counter'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 9),
        // property 'truncationOffset'
        QtMocHelpers::PropertyData<uint>(19, QMetaType::UInt, QMC::DefaultPropertyFlags, 10),
        // property 'fixedTruncation'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TOTPAlgorithm'
        QtMocHelpers::EnumData<enum TOTPAlgorithm>(33, 33, QMC::EnumFlags{}).add({
            {   34, TOTPAlgorithm::Sha1 },
            {   35, TOTPAlgorithm::Sha256 },
            {   36, TOTPAlgorithm::Sha512 },
        }),
        // enum 'TokenType'
        QtMocHelpers::EnumData<enum TokenType>(37, 37, QMC::EnumFlags{}).add({
            {   38, TokenType::Hotp },
            {   39, TokenType::Totp },
        }),
    };
    return QtMocHelpers::metaObjectData<AccountInput, qt_meta_tag_ZN5model12AccountInputE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject model::AccountInput::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5model12AccountInputE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5model12AccountInputE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5model12AccountInputE_t>.metaTypes,
    nullptr
} };

void model::AccountInput::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountInput *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->typeChanged(); break;
        case 1: _t->nameChanged(); break;
        case 2: _t->issuerChanged(); break;
        case 3: _t->secretChanged(); break;
        case 4: _t->tokenLengthChanged(); break;
        case 5: _t->timeStepChanged(); break;
        case 6: _t->algorithmChanged(); break;
        case 7: _t->epochChanged(); break;
        case 8: _t->checksumChanged(); break;
        case 9: _t->counterChanged(); break;
        case 10: _t->truncationChanged(); break;
        case 11: _t->reset(); break;
        case 12: _t->setCounter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<validators::UnsignedLongValidator*>>(_a[2]))); break;
        case 13: _t->setTruncationOffset((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 14: _t->setDynamicTruncation(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< validators::UnsignedLongValidator* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::typeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::nameChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::issuerChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::secretChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::tokenLengthChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::timeStepChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::algorithmChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::epochChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::checksumChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::counterChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountInput::*)()>(_a, &AccountInput::truncationChanged, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<model::AccountInput::TokenType*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->issuer(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->secret(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->tokenLength(); break;
        case 5: *reinterpret_cast<uint*>(_v) = _t->timeStep(); break;
        case 6: *reinterpret_cast<model::AccountInput::TOTPAlgorithm*>(_v) = _t->algorithm(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->epoch(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->checksum(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->counter(); break;
        case 10: *reinterpret_cast<uint*>(_v) = _t->truncationOffset(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->fixedTruncation(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setType(*reinterpret_cast<model::AccountInput::TokenType*>(_v)); break;
        case 1: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setIssuer(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setSecret(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setTokenLength(*reinterpret_cast<uint*>(_v)); break;
        case 5: _t->setTimeStep(*reinterpret_cast<uint*>(_v)); break;
        case 6: _t->setAlgorithm(*reinterpret_cast<model::AccountInput::TOTPAlgorithm*>(_v)); break;
        case 7: _t->setEpoch(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setChecksum(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *model::AccountInput::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *model::AccountInput::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5model12AccountInputE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int model::AccountInput::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void model::AccountInput::typeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void model::AccountInput::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void model::AccountInput::issuerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void model::AccountInput::secretChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void model::AccountInput::tokenLengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void model::AccountInput::timeStepChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void model::AccountInput::algorithmChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void model::AccountInput::epochChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void model::AccountInput::checksumChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void model::AccountInput::counterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void model::AccountInput::truncationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
namespace {
struct qt_meta_tag_ZN5model11ImportInputE_t {};
} // unnamed namespace

template <> constexpr inline auto model::ImportInput::qt_create_metaobjectdata<qt_meta_tag_ZN5model11ImportInputE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "model::ImportInput",
        "fileChanged",
        "",
        "formatChanged",
        "passwordChanged",
        "reset",
        "file",
        "format",
        "model::ImportInput::ImportFormat",
        "password",
        "ImportFormat",
        "FreeOTPURIs",
        "AndOTPPlainJSON",
        "AndOTPEncryptedJSON",
        "AegisPlainJSON",
        "AegisEncryptedJSON",
        "SailOTPDecryptedJson"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fileChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'formatChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'passwordChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'reset'
        QtMocHelpers::MethodData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'file'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'format'
        QtMocHelpers::PropertyData<model::ImportInput::ImportFormat>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'password'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ImportFormat'
        QtMocHelpers::EnumData<enum ImportFormat>(10, 10, QMC::EnumFlags{}).add({
            {   11, ImportFormat::FreeOTPURIs },
            {   12, ImportFormat::AndOTPPlainJSON },
            {   13, ImportFormat::AndOTPEncryptedJSON },
            {   14, ImportFormat::AegisPlainJSON },
            {   15, ImportFormat::AegisEncryptedJSON },
            {   16, ImportFormat::SailOTPDecryptedJson },
        }),
    };
    return QtMocHelpers::metaObjectData<ImportInput, qt_meta_tag_ZN5model11ImportInputE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject model::ImportInput::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5model11ImportInputE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5model11ImportInputE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5model11ImportInputE_t>.metaTypes,
    nullptr
} };

void model::ImportInput::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ImportInput *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileChanged(); break;
        case 1: _t->formatChanged(); break;
        case 2: _t->passwordChanged(); break;
        case 3: _t->reset(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ImportInput::*)()>(_a, &ImportInput::fileChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportInput::*)()>(_a, &ImportInput::formatChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportInput::*)()>(_a, &ImportInput::passwordChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->file(); break;
        case 1: *reinterpret_cast<model::ImportInput::ImportFormat*>(_v) = _t->format(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->password(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFile(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setFormat(*reinterpret_cast<model::ImportInput::ImportFormat*>(_v)); break;
        case 2: _t->setPassword(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *model::ImportInput::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *model::ImportInput::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5model11ImportInputE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int model::ImportInput::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void model::ImportInput::fileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void model::ImportInput::formatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void model::ImportInput::passwordChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
