/****************************************************************************
** Meta object code from reading C++ file 'account.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../keysmith-26.07.80/src/account/account.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'account.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8accounts7AccountE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::Account::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts7AccountE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::Account",
        "tokenChanged",
        "",
        "otp",
        "removed",
        "updated",
        "remove",
        "recompute",
        "setCounter",
        "counter",
        "advanceCounter",
        "by",
        "Algorithm",
        "Totp",
        "Hotp",
        "Hash",
        "Sha1",
        "Sha256",
        "Sha512"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'tokenChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'removed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updated'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'remove'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'recompute'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCounter'
        QtMocHelpers::SlotData<void(quint64)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 9 },
        }}),
        // Slot 'advanceCounter'
        QtMocHelpers::SlotData<void(quint64)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 11 },
        }}),
        // Slot 'advanceCounter'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Algorithm'
        QtMocHelpers::EnumData<enum Algorithm>(12, 12, QMC::EnumFlags{}).add({
            {   13, Algorithm::Totp },
            {   14, Algorithm::Hotp },
        }),
        // enum 'Hash'
        QtMocHelpers::EnumData<enum Hash>(15, 15, QMC::EnumFlags{}).add({
            {   16, Hash::Sha1 },
            {   17, Hash::Sha256 },
            {   18, Hash::Sha512 },
        }),
    };
    return QtMocHelpers::metaObjectData<Account, qt_meta_tag_ZN8accounts7AccountE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::Account::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts7AccountE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts7AccountE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts7AccountE_t>.metaTypes,
    nullptr
} };

void accounts::Account::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Account *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tokenChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->removed(); break;
        case 2: _t->updated(); break;
        case 3: _t->remove(); break;
        case 4: _t->recompute(); break;
        case 5: _t->setCounter((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 6: _t->advanceCounter((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 7: _t->advanceCounter(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Account::*)(const QString & )>(_a, &Account::tokenChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::removed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::updated, 2))
            return;
    }
}

const QMetaObject *accounts::Account::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::Account::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts7AccountE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int accounts::Account::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void accounts::Account::tokenChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void accounts::Account::removed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void accounts::Account::updated()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN8accounts14AccountStorageE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::AccountStorage::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts14AccountStorageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::AccountStorage",
        "added",
        "",
        "fullName",
        "removed",
        "error",
        "loaded",
        "disposed",
        "unlock",
        "load",
        "accountRemoved",
        "handleDisposal",
        "handleError",
        "handleLoaded",
        "handleHotp",
        "QUuid",
        "id",
        "name",
        "issuer",
        "secret",
        "nonce",
        "tokenLength",
        "counter",
        "fixedTruncation",
        "offset",
        "checksum",
        "handleTotp",
        "timeStep",
        "epoch",
        "Account::Hash",
        "hash"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'added'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'removed'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loaded'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disposed'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'unlock'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'load'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountRemoved'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleDisposal'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleError'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleLoaded'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleHotp'
        QtMocHelpers::SlotData<void(const QUuid, const QString &, const QString &, const QByteArray &, const QByteArray &, uint, quint64, bool, uint, bool)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QByteArray, 19 },
            { QMetaType::QByteArray, 20 }, { QMetaType::UInt, 21 }, { QMetaType::ULongLong, 22 }, { QMetaType::Bool, 23 },
            { QMetaType::UInt, 24 }, { QMetaType::Bool, 25 },
        }}),
        // Slot 'handleTotp'
        QtMocHelpers::SlotData<void(const QUuid, const QString &, const QString &, const QByteArray &, const QByteArray &, uint, uint, const QDateTime &, Account::Hash)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QByteArray, 19 },
            { QMetaType::QByteArray, 20 }, { QMetaType::UInt, 21 }, { QMetaType::UInt, 27 }, { QMetaType::QDateTime, 28 },
            { 0x80000000 | 29, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountStorage, qt_meta_tag_ZN8accounts14AccountStorageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::AccountStorage::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts14AccountStorageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts14AccountStorageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts14AccountStorageE_t>.metaTypes,
    nullptr
} };

void accounts::AccountStorage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountStorage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->added((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->removed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->error(); break;
        case 3: _t->loaded(); break;
        case 4: _t->disposed(); break;
        case 5: _t->unlock(); break;
        case 6: _t->load(); break;
        case 7: _t->accountRemoved(); break;
        case 8: _t->handleDisposal(); break;
        case 9: _t->handleError(); break;
        case 10: _t->handleLoaded(); break;
        case 11: _t->handleHotp((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[10]))); break;
        case 12: _t->handleTotp((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<Account::Hash>>(_a[9]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountStorage::*)(const QString & )>(_a, &AccountStorage::added, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountStorage::*)(const QString & )>(_a, &AccountStorage::removed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountStorage::*)()>(_a, &AccountStorage::error, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountStorage::*)()>(_a, &AccountStorage::loaded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountStorage::*)()>(_a, &AccountStorage::disposed, 4))
            return;
    }
}

const QMetaObject *accounts::AccountStorage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::AccountStorage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts14AccountStorageE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int accounts::AccountStorage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void accounts::AccountStorage::added(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void accounts::AccountStorage::removed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void accounts::AccountStorage::error()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void accounts::AccountStorage::loaded()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void accounts::AccountStorage::disposed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
