/****************************************************************************
** Meta object code from reading C++ file 'actions_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../keysmith-26.07.80/src/account/actions_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'actions_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8accounts10AccountJobE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::AccountJob::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts10AccountJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::AccountJob",
        "finished",
        "",
        "run"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'run'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountJob, qt_meta_tag_ZN8accounts10AccountJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::AccountJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts10AccountJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts10AccountJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts10AccountJobE_t>.metaTypes,
    nullptr
} };

void accounts::AccountJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished(); break;
        case 1: _t->run(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountJob::*)()>(_a, &AccountJob::finished, 0))
            return;
    }
}

const QMetaObject *accounts::AccountJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::AccountJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts10AccountJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int accounts::AccountJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void accounts::AccountJob::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN8accounts4NullE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::Null::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts4NullE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::Null"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Null, qt_meta_tag_ZN8accounts4NullE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::Null::staticMetaObject = { {
    QMetaObject::SuperData::link<AccountJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts4NullE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts4NullE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts4NullE_t>.metaTypes,
    nullptr
} };

void accounts::Null::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Null *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *accounts::Null::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::Null::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts4NullE_t>.strings))
        return static_cast<void*>(this);
    return AccountJob::qt_metacast(_clname);
}

int accounts::Null::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AccountJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8accounts22RequestAccountPasswordE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::RequestAccountPassword::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts22RequestAccountPasswordE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::RequestAccountPassword",
        "unlocked",
        "",
        "failed",
        "fail",
        "unlock",
        "finish"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'unlocked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'failed'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fail'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'unlock'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'finish'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RequestAccountPassword, qt_meta_tag_ZN8accounts22RequestAccountPasswordE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::RequestAccountPassword::staticMetaObject = { {
    QMetaObject::SuperData::link<AccountJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts22RequestAccountPasswordE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts22RequestAccountPasswordE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts22RequestAccountPasswordE_t>.metaTypes,
    nullptr
} };

void accounts::RequestAccountPassword::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RequestAccountPassword *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->unlocked(); break;
        case 1: _t->failed(); break;
        case 2: _t->fail(); break;
        case 3: _t->unlock(); break;
        case 4: _t->finish(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RequestAccountPassword::*)()>(_a, &RequestAccountPassword::unlocked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RequestAccountPassword::*)()>(_a, &RequestAccountPassword::failed, 1))
            return;
    }
}

const QMetaObject *accounts::RequestAccountPassword::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::RequestAccountPassword::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts22RequestAccountPasswordE_t>.strings))
        return static_cast<void*>(this);
    return AccountJob::qt_metacast(_clname);
}

int accounts::RequestAccountPassword::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AccountJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void accounts::RequestAccountPassword::unlocked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void accounts::RequestAccountPassword::failed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN8accounts12LoadAccountsE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::LoadAccounts::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts12LoadAccountsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::LoadAccounts",
        "foundHotp",
        "",
        "QUuid",
        "id",
        "name",
        "issuer",
        "secret",
        "nonce",
        "tokenLength",
        "counter",
        "fixedTruncation",
        "offset",
        "checksum",
        "foundTotp",
        "timeStep",
        "epoch",
        "accounts::Account::Hash",
        "hash",
        "failedToLoadAllAccounts"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'foundHotp'
        QtMocHelpers::SignalData<void(const QUuid, const QString, const QString, const QByteArray, const QByteArray, uint, quint64, bool, uint, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 }, { QMetaType::QByteArray, 7 },
            { QMetaType::QByteArray, 8 }, { QMetaType::UInt, 9 }, { QMetaType::ULongLong, 10 }, { QMetaType::Bool, 11 },
            { QMetaType::UInt, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Signal 'foundTotp'
        QtMocHelpers::SignalData<void(const QUuid, const QString, const QString, const QByteArray, const QByteArray, uint, uint, const QDateTime, accounts::Account::Hash)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 }, { QMetaType::QByteArray, 7 },
            { QMetaType::QByteArray, 8 }, { QMetaType::UInt, 9 }, { QMetaType::UInt, 15 }, { QMetaType::QDateTime, 16 },
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'failedToLoadAllAccounts'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LoadAccounts, qt_meta_tag_ZN8accounts12LoadAccountsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::LoadAccounts::staticMetaObject = { {
    QMetaObject::SuperData::link<AccountJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts12LoadAccountsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts12LoadAccountsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts12LoadAccountsE_t>.metaTypes,
    nullptr
} };

void accounts::LoadAccounts::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LoadAccounts *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->foundHotp((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[10]))); break;
        case 1: _t->foundTotp((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<accounts::Account::Hash>>(_a[9]))); break;
        case 2: _t->failedToLoadAllAccounts(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LoadAccounts::*)(const QUuid , const QString , const QString , const QByteArray , const QByteArray , uint , quint64 , bool , uint , bool )>(_a, &LoadAccounts::foundHotp, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoadAccounts::*)(const QUuid , const QString , const QString , const QByteArray , const QByteArray , uint , uint , const QDateTime , accounts::Account::Hash )>(_a, &LoadAccounts::foundTotp, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoadAccounts::*)()>(_a, &LoadAccounts::failedToLoadAllAccounts, 2))
            return;
    }
}

const QMetaObject *accounts::LoadAccounts::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::LoadAccounts::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts12LoadAccountsE_t>.strings))
        return static_cast<void*>(this);
    return AccountJob::qt_metacast(_clname);
}

int accounts::LoadAccounts::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AccountJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void accounts::LoadAccounts::foundHotp(const QUuid _t1, const QString _t2, const QString _t3, const QByteArray _t4, const QByteArray _t5, uint _t6, quint64 _t7, bool _t8, uint _t9, bool _t10)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10);
}

// SIGNAL 1
void accounts::LoadAccounts::foundTotp(const QUuid _t1, const QString _t2, const QString _t3, const QByteArray _t4, const QByteArray _t5, uint _t6, uint _t7, const QDateTime _t8, accounts::Account::Hash _t9)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9);
}

// SIGNAL 2
void accounts::LoadAccounts::failedToLoadAllAccounts()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN8accounts14DeleteAccountsE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::DeleteAccounts::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts14DeleteAccountsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::DeleteAccounts",
        "invalid",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'invalid'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DeleteAccounts, qt_meta_tag_ZN8accounts14DeleteAccountsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::DeleteAccounts::staticMetaObject = { {
    QMetaObject::SuperData::link<AccountJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts14DeleteAccountsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts14DeleteAccountsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts14DeleteAccountsE_t>.metaTypes,
    nullptr
} };

void accounts::DeleteAccounts::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeleteAccounts *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->invalid(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeleteAccounts::*)()>(_a, &DeleteAccounts::invalid, 0))
            return;
    }
}

const QMetaObject *accounts::DeleteAccounts::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::DeleteAccounts::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts14DeleteAccountsE_t>.strings))
        return static_cast<void*>(this);
    return AccountJob::qt_metacast(_clname);
}

int accounts::DeleteAccounts::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AccountJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void accounts::DeleteAccounts::invalid()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN8accounts8SaveHotpE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::SaveHotp::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts8SaveHotpE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::SaveHotp",
        "invalid",
        "",
        "saved",
        "QUuid",
        "id",
        "accountName",
        "issuer",
        "secret",
        "nonce",
        "tokenLength",
        "counter",
        "fixedTruncation",
        "offset",
        "checksum"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'invalid'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saved'
        QtMocHelpers::SignalData<void(const QUuid, const QString, const QString, const QByteArray, const QByteArray, uint, quint64, bool, uint, bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { QMetaType::QByteArray, 8 },
            { QMetaType::QByteArray, 9 }, { QMetaType::UInt, 10 }, { QMetaType::ULongLong, 11 }, { QMetaType::Bool, 12 },
            { QMetaType::UInt, 13 }, { QMetaType::Bool, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SaveHotp, qt_meta_tag_ZN8accounts8SaveHotpE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::SaveHotp::staticMetaObject = { {
    QMetaObject::SuperData::link<AccountJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts8SaveHotpE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts8SaveHotpE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts8SaveHotpE_t>.metaTypes,
    nullptr
} };

void accounts::SaveHotp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SaveHotp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->invalid(); break;
        case 1: _t->saved((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[10]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SaveHotp::*)()>(_a, &SaveHotp::invalid, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SaveHotp::*)(const QUuid , const QString , const QString , const QByteArray , const QByteArray , uint , quint64 , bool , uint , bool )>(_a, &SaveHotp::saved, 1))
            return;
    }
}

const QMetaObject *accounts::SaveHotp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::SaveHotp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts8SaveHotpE_t>.strings))
        return static_cast<void*>(this);
    return AccountJob::qt_metacast(_clname);
}

int accounts::SaveHotp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AccountJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void accounts::SaveHotp::invalid()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void accounts::SaveHotp::saved(const QUuid _t1, const QString _t2, const QString _t3, const QByteArray _t4, const QByteArray _t5, uint _t6, quint64 _t7, bool _t8, uint _t9, bool _t10)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10);
}
namespace {
struct qt_meta_tag_ZN8accounts8SaveTotpE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::SaveTotp::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts8SaveTotpE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::SaveTotp",
        "invalid",
        "",
        "saved",
        "QUuid",
        "id",
        "accountName",
        "issuer",
        "secret",
        "nonce",
        "tokenLength",
        "timeStep",
        "epoch",
        "accounts::Account::Hash",
        "hash"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'invalid'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saved'
        QtMocHelpers::SignalData<void(const QUuid, const QString, const QString, const QByteArray, const QByteArray, uint, uint, const QDateTime, accounts::Account::Hash)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { QMetaType::QByteArray, 8 },
            { QMetaType::QByteArray, 9 }, { QMetaType::UInt, 10 }, { QMetaType::UInt, 11 }, { QMetaType::QDateTime, 12 },
            { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SaveTotp, qt_meta_tag_ZN8accounts8SaveTotpE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::SaveTotp::staticMetaObject = { {
    QMetaObject::SuperData::link<AccountJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts8SaveTotpE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts8SaveTotpE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts8SaveTotpE_t>.metaTypes,
    nullptr
} };

void accounts::SaveTotp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SaveTotp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->invalid(); break;
        case 1: _t->saved((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<accounts::Account::Hash>>(_a[9]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SaveTotp::*)()>(_a, &SaveTotp::invalid, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SaveTotp::*)(const QUuid , const QString , const QString , const QByteArray , const QByteArray , uint , uint , const QDateTime , accounts::Account::Hash )>(_a, &SaveTotp::saved, 1))
            return;
    }
}

const QMetaObject *accounts::SaveTotp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::SaveTotp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts8SaveTotpE_t>.strings))
        return static_cast<void*>(this);
    return AccountJob::qt_metacast(_clname);
}

int accounts::SaveTotp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AccountJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void accounts::SaveTotp::invalid()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void accounts::SaveTotp::saved(const QUuid _t1, const QString _t2, const QString _t3, const QByteArray _t4, const QByteArray _t5, uint _t6, uint _t7, const QDateTime _t8, accounts::Account::Hash _t9)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9);
}
namespace {
struct qt_meta_tag_ZN8accounts11ComputeTotpE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::ComputeTotp::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts11ComputeTotpE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::ComputeTotp",
        "otp",
        "",
        "nextOtp",
        "validFrom",
        "validUntil"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'otp'
        QtMocHelpers::SignalData<void(const QString, const QString, const QDateTime, const QDateTime)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 1 }, { QMetaType::QString, 3 }, { QMetaType::QDateTime, 4 }, { QMetaType::QDateTime, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ComputeTotp, qt_meta_tag_ZN8accounts11ComputeTotpE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::ComputeTotp::staticMetaObject = { {
    QMetaObject::SuperData::link<AccountJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts11ComputeTotpE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts11ComputeTotpE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts11ComputeTotpE_t>.metaTypes,
    nullptr
} };

void accounts::ComputeTotp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ComputeTotp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->otp((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ComputeTotp::*)(const QString , const QString , const QDateTime , const QDateTime )>(_a, &ComputeTotp::otp, 0))
            return;
    }
}

const QMetaObject *accounts::ComputeTotp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::ComputeTotp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts11ComputeTotpE_t>.strings))
        return static_cast<void*>(this);
    return AccountJob::qt_metacast(_clname);
}

int accounts::ComputeTotp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AccountJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void accounts::ComputeTotp::otp(const QString _t1, const QString _t2, const QDateTime _t3, const QDateTime _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}
namespace {
struct qt_meta_tag_ZN8accounts11ComputeHotpE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::ComputeHotp::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts11ComputeHotpE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::ComputeHotp",
        "otp",
        "",
        "nextOtp",
        "nextCounter"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'otp'
        QtMocHelpers::SignalData<void(const QString, const QString, const quint64)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 1 }, { QMetaType::QString, 3 }, { QMetaType::ULongLong, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ComputeHotp, qt_meta_tag_ZN8accounts11ComputeHotpE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::ComputeHotp::staticMetaObject = { {
    QMetaObject::SuperData::link<AccountJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts11ComputeHotpE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts11ComputeHotpE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts11ComputeHotpE_t>.metaTypes,
    nullptr
} };

void accounts::ComputeHotp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ComputeHotp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->otp((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ComputeHotp::*)(const QString , const QString , const quint64 )>(_a, &ComputeHotp::otp, 0))
            return;
    }
}

const QMetaObject *accounts::ComputeHotp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::ComputeHotp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts11ComputeHotpE_t>.strings))
        return static_cast<void*>(this);
    return AccountJob::qt_metacast(_clname);
}

int accounts::ComputeHotp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AccountJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void accounts::ComputeHotp::otp(const QString _t1, const QString _t2, const quint64 _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN8accounts10DispatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto accounts::Dispatcher::qt_create_metaobjectdata<qt_meta_tag_ZN8accounts10DispatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "accounts::Dispatcher",
        "dispatch",
        "",
        "next"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dispatch'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'next'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Dispatcher, qt_meta_tag_ZN8accounts10DispatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject accounts::Dispatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts10DispatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts10DispatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8accounts10DispatcherE_t>.metaTypes,
    nullptr
} };

void accounts::Dispatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Dispatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dispatch(); break;
        case 1: _t->next(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Dispatcher::*)()>(_a, &Dispatcher::dispatch, 0))
            return;
    }
}

const QMetaObject *accounts::Dispatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *accounts::Dispatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8accounts10DispatcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int accounts::Dispatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void accounts::Dispatcher::dispatch()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
